@@ -21,6 +21,7 @@ import java.lang.reflect.Array;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
@@ -75,7 +76,7 @@ import org.apache.commons.logging.impl.NoOpLog;
  * @author <a href="mailto:ksh@scand.com">Konstantin Shaposhnikov </a>
  * @author Oliver Heger
  * @author <a href="mailto:hps@intermeta.de">Henning P. Schmiedehausen </a>
- * @version $Id: AbstractConfiguration.java 589138 2007-10-27 15:41:13Z oheger $
+ * @version $Id: AbstractConfiguration.java 652316 2008-04-30 10:59:58Z ebourg $
  */
 public abstract class AbstractConfiguration extends EventSource implements Configuration
 {
@@ -1134,6 +1135,10 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         {
             list = defaultValue;
         }
+        else if (value.getClass().isArray())
+        {
+            return Arrays.asList((Object[]) value);
+        }
         else
         {
             throw new ConversionException('\'' + key + "' doesn't map to a List object: " + value + ", a "
