@@ -17,11 +17,11 @@
 
 package org.apache.commons.configuration;
 
-import java.io.Reader;
-import java.io.Writer;
 import java.io.File;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.Reader;
+import java.io.Writer;
 import java.net.URL;
 import java.util.Collection;
 import java.util.Iterator;
@@ -41,7 +41,7 @@ import org.apache.commons.configuration.reloading.ReloadingStrategy;
  * @since 1.2
  *
  * @author Emmanuel Bourg
- * @version $Revision: 589380 $, $Date: 2007-10-28 17:37:35 +0100 (So, 28 Okt 2007) $
+ * @version $Revision: 712405 $, $Date: 2008-11-08 17:37:48 +0100 (Sa, 08 Nov 2008) $
  */
 public abstract class AbstractHierarchicalFileConfiguration
 extends HierarchicalConfiguration
@@ -308,6 +308,12 @@ implements FileConfiguration, ConfigurationListener
         return super.containsKey(key);
     }
 
+    public Iterator getKeys()
+    {
+        reload();
+        return super.getKeys();
+    }
+
     public Iterator getKeys(String prefix)
     {
         reload();
