@@ -36,7 +36,7 @@ import java.util.NoSuchElementException;
  * each part it can be checked whether it has an associated index.</p>
  *
  * @author <a href="mailto:oliver.heger@t-online.de">Oliver Heger</a>
- * @version $Id: ConfigurationKey.java 439648 2006-09-02 20:42:10Z oheger $
+ * @version $Id: ConfigurationKey.java 726809 2008-12-15 21:29:56Z oheger $
  */
 public class ConfigurationKey implements Serializable
 {
@@ -211,7 +211,7 @@ public class ConfigurationKey implements Serializable
         {
             count++;
         }
-        return count % 2 == 1;
+        return count % 2 != 0;
     }
 
     /**
