@@ -33,7 +33,9 @@ import java.util.Iterator;
 import org.apache.commons.configuration.event.ConfigurationErrorEvent;
 import org.apache.commons.configuration.event.ConfigurationErrorListener;
 import org.apache.commons.configuration.event.EventSource;
+import org.apache.commons.configuration.tree.ExpressionEngine;
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang.SystemUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -45,7 +47,7 @@ import org.apache.commons.logging.LogFactory;
  * @author <a href="mailto:herve.quiroz@esil.univ-mrs.fr">Herve Quiroz</a>
  * @author <a href="mailto:oliver.heger@t-online.de">Oliver Heger</a>
  * @author Emmanuel Bourg
- * @version $Revision: 567771 $, $Date: 2007-08-20 19:57:08 +0200 (Mo, 20 Aug 2007) $
+ * @version $Revision: 720600 $, $Date: 2008-11-25 22:20:01 +0100 (Di, 25 Nov 2008) $
  */
 public final class ConfigurationUtils
 {
@@ -58,6 +60,9 @@ public final class ConfigurationUtils
     /** Constant for the name of the clone() method.*/
     private static final String METHOD_CLONE = "clone";
 
+    /** Constant for Java version 1.4.*/
+    private static final float JAVA_1_4 = 1.4f;
+
     /** The logger.*/
     private static Log log = LogFactory.getLog(ConfigurationUtils.class);
 
@@ -66,7 +71,7 @@ public final class ConfigurationUtils
      */
     private ConfigurationUtils()
     {
-        // to prevent instanciation...
+        // to prevent instantiation...
     }
 
     /**
@@ -180,6 +185,35 @@ public final class ConfigurationUtils
      */
     public static HierarchicalConfiguration convertToHierarchical(
             Configuration conf)
+    {
+        return convertToHierarchical(conf, null);
+    }
+
+    /**
+     * Converts the passed in <code>Configuration</code> object to a
+     * hierarchical one using the specified <code>ExpressionEngine</code>. This
+     * conversion works by adding the keys found in the configuration to a newly
+     * created hierarchical configuration. When adding new keys to a
+     * hierarchical configuration the keys are interpreted by its
+     * <code>ExpressionEngine</code>. If they contain special characters (e.g.
+     * brackets) that are treated in a special way by the default expression
+     * engine, it may be necessary using a specific engine that can deal with
+     * such characters. Otherwise <b>null</b> can be passed in for the
+     * <code>ExpressionEngine</code>; then the default expression engine is
+     * used. If the passed in configuration is already hierarchical, it is
+     * directly returned. (However, the <code>ExpressionEngine</code> is set if
+     * it is not <b>null</b>.) Otherwise all properties are copied into a new
+     * hierarchical configuration.
+     *
+     * @param conf the configuration to convert
+     * @param engine the <code>ExpressionEngine</code> for the hierarchical
+     *        configuration or <b>null</b> for the default
+     * @return the new hierarchical configuration (the result is <b>null</b> if
+     *         and only if the passed in configuration is <b>null</b>)
+     * @since 1.6
+     */
+    public static HierarchicalConfiguration convertToHierarchical(
+            Configuration conf, ExpressionEngine engine)
     {
         if (conf == null)
         {
@@ -188,15 +222,26 @@ public final class ConfigurationUtils
 
         if (conf instanceof HierarchicalConfiguration)
         {
-            return (HierarchicalConfiguration) conf;
+            HierarchicalConfiguration hc = (HierarchicalConfiguration) conf;
+            if (engine != null)
+            {
+                hc.setExpressionEngine(engine);
+            }
+
+            return hc;
         }
         else
         {
             HierarchicalConfiguration hc = new HierarchicalConfiguration();
+            if (engine != null)
+            {
+                hc.setExpressionEngine(engine);
+            }
+
             // Workaround for problem with copy()
             boolean delimiterParsingStatus = hc.isDelimiterParsingDisabled();
             hc.setDelimiterParsingDisabled(true);
-            ConfigurationUtils.copy(conf, hc);
+            hc.append(conf);
             hc.setDelimiterParsingDisabled(delimiterParsingStatus);
             return hc;
         }
@@ -294,7 +339,7 @@ public final class ConfigurationUtils
         File f = new File(file);
         if (f.isAbsolute()) // already absolute?
         {
-            return f.toURL();
+            return toURL(f);
         }
 
         try
@@ -311,7 +356,7 @@ public final class ConfigurationUtils
         }
         catch (MalformedURLException uex)
         {
-            return constructFile(basePath, file).toURL();
+            return toURL(constructFile(basePath, file));
         }
     }
 
@@ -452,7 +497,7 @@ public final class ConfigurationUtils
             {
                 try
                 {
-                    url = file.toURL();
+                    url = toURL(file);
                     log.debug("Loading configuration from the absolute path " + name);
                 }
                 catch (MalformedURLException e)
@@ -470,7 +515,7 @@ public final class ConfigurationUtils
                 File file = constructFile(base, name);
                 if (file != null && file.exists())
                 {
-                    url = file.toURL();
+                    url = toURL(file);
                 }
 
                 if (url != null)
@@ -492,7 +537,7 @@ public final class ConfigurationUtils
                 File file = constructFile(System.getProperty("user.home"), name);
                 if (file != null && file.exists())
                 {
-                    url = file.toURL();
+                    url = toURL(file);
                 }
 
                 if (url != null)
@@ -679,6 +724,40 @@ public final class ConfigurationUtils
         }
     }
 
+    /**
+     * Convert the specified file into an URL. This method is equivalent
+     * to file.toURI().toURL() on Java 1.4 and above, and equivalent to
+     * file.toURL() on Java 1.3. This is to work around a bug in the JDK
+     * preventing the transformation of a file into an URL if the file name
+     * contains a '#' character. See the issue CONFIGURATION-300 for
+     * more details.
+     *
+     * @param file the file to be converted into an URL
+     */
+    static URL toURL(File file) throws MalformedURLException
+    {
+        if (SystemUtils.isJavaVersionAtLeast(JAVA_1_4))
+        {
+            try
+            {
+                Method toURI = file.getClass().getMethod("toURI", (Class[]) null);
+                Object uri = toURI.invoke(file, (Class[]) null);
+                Method toURL = uri.getClass().getMethod("toURL", (Class[]) null);
+                URL url = (URL) toURL.invoke(uri, (Class[]) null);
+
+                return url;
+            }
+            catch (Exception e)
+            {
+                throw new MalformedURLException(e.getMessage());
+            }
+        }
+        else
+        {
+            return file.toURL();
+        }
+    }
+
     /**
      * Enables runtime exceptions for the specified configuration object. This
      * method can be used for configuration implementations that may face errors
