@@ -26,12 +26,11 @@ import java.util.List;
 import java.util.Set;
 import java.util.Stack;
 
-import org.apache.commons.collections.set.ListOrderedSet;
 import org.apache.commons.collections.iterators.SingletonIterator;
+import org.apache.commons.collections.set.ListOrderedSet;
 import org.apache.commons.configuration.event.ConfigurationEvent;
 import org.apache.commons.configuration.event.ConfigurationListener;
 import org.apache.commons.configuration.tree.ConfigurationNode;
-import org.apache.commons.configuration.tree.ConfigurationNodeVisitor;
 import org.apache.commons.configuration.tree.ConfigurationNodeVisitorAdapter;
 import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
@@ -124,7 +123,7 @@ import org.apache.commons.lang.StringUtils;
  * synchronization has to be performed manually.</p>
  *
  * @author Oliver Heger
- * @version $Id: HierarchicalConfiguration.java 593319 2007-11-08 21:14:21Z oheger $
+ * @version $Id: HierarchicalConfiguration.java 722238 2008-12-01 21:28:31Z oheger $
  */
 public class HierarchicalConfiguration extends AbstractConfiguration implements Serializable, Cloneable
 {
@@ -203,6 +202,12 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      */
     public Node getRoot()
     {
+        if (root == null && rootNode != null)
+        {
+            // Dynamically create a snapshot of the root node
+            return new Node(rootNode);
+        }
+
         return root;
     }
 
@@ -250,7 +255,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         this.rootNode = rootNode;
 
         // For backward compatibility also set the old root field.
-        root = (rootNode instanceof Node) ? (Node) rootNode : new Node(rootNode);
+        root = (rootNode instanceof Node) ? (Node) rootNode : null;
     }
 
     /**
@@ -417,16 +422,6 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
                     "Cannot add nodes to an attribute node!");
         }
 
-        // a visitor to ensure that the nodes' references are cleared; this is
-        // necessary if the nodes are moved from another configuration
-        ConfigurationNodeVisitor clearRefVisitor = new ConfigurationNodeVisitorAdapter()
-        {
-            public void visitBeforeChildren(ConfigurationNode node)
-            {
-                node.setReference(null);
-            }
-        };
-
         for (Iterator it = nodes.iterator(); it.hasNext();)
         {
             ConfigurationNode child = (ConfigurationNode) it.next();
@@ -438,7 +433,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
             {
                 parent.addChild(child);
             }
-            child.visit(clearRefVisitor);
+            clearReferences(child);
         }
         fireEvent(EVENT_ADD_NODES, key, nodes, false);
     }
@@ -829,6 +824,12 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
     public Iterator getKeys(String prefix)
     {
         DefinedKeysVisitor visitor = new DefinedKeysVisitor(prefix);
+        if (containsKey(prefix))
+        {
+            // explicitly add the prefix
+            visitor.getKeyList().add(prefix);
+        }
+
         List nodes = fetchNodeList(prefix);
 
         for (Iterator itNodes = nodes.iterator(); itNodes.hasNext();)
@@ -1213,11 +1214,20 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
             setReference(src.getReference());
             for (Iterator it = src.getChildren().iterator(); it.hasNext();)
             {
-                addChild((ConfigurationNode) it.next());
+                ConfigurationNode nd = (ConfigurationNode) it.next();
+                // Don't change the parent node
+                ConfigurationNode parent = nd.getParentNode();
+                addChild(nd);
+                nd.setParentNode(parent);
             }
+
             for (Iterator it = src.getAttributes().iterator(); it.hasNext();)
             {
-                addAttribute((ConfigurationNode) it.next());
+                ConfigurationNode nd = (ConfigurationNode) it.next();
+                // Don't change the parent node
+                ConfigurationNode parent = nd.getParentNode();
+                addAttribute(nd);
+                nd.setParentNode(parent);
             }
         }
 
