@@ -33,7 +33,7 @@ import org.apache.commons.lang.StringUtils;
 
 /**
  * <p>
- * An initialization or ini file is a configuration file tpically found on
+ * An initialization or ini file is a configuration file typically found on
  * Microsoft's Windows operating system and contains data for Windows based
  * applications.
  * </p>
@@ -158,14 +158,16 @@ import org.apache.commons.lang.StringUtils;
  * <code>getSections</code> method.
  * </p>
  * <p>
- * <em>Note:</em>Configuration objects of this type can be read concurrently
+ * <em>Note:</em> Configuration objects of this type can be read concurrently
  * by multiple threads. However if one of these threads modifies the object,
  * synchronization has to be performed manually.
  * </p>
  *
  * @author Trevor Miller
- * @version $Id: INIConfiguration.java 589380 2007-10-28 16:37:35Z oheger $
+ * @version $Id: INIConfiguration.java 720600 2008-11-25 21:20:01Z oheger $
  * @since 1.4
+ * @deprecated This class has been replaced by HierarchicalINIConfiguration,
+ * which provides a superset of the functionality offered by this class.
  */
 public class INIConfiguration extends AbstractFileConfiguration
 {
@@ -326,9 +328,9 @@ public class INIConfiguration extends AbstractFileConfiguration
                 line = bufferedReader.readLine();
             }
         }
-        catch (IOException ioe)
+        catch (IOException e)
         {
-            throw new ConfigurationException(ioe.getMessage());
+            throw new ConfigurationException("Unable to load the configuration", e);
         }
     }
 
@@ -400,7 +402,12 @@ public class INIConfiguration extends AbstractFileConfiguration
             i++;
         }
 
-        return result.toString().trim();
+        String v = result.toString();
+        if (!quoted)
+        {
+            v = v.trim();
+        }
+        return v;
     }
 
     /**
