@@ -17,23 +17,24 @@
 
 package org.apache.commons.configuration;
 
-import java.net.URL;
-import java.util.Properties;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.net.URL;
+import java.util.Iterator;
+import java.util.Properties;
+
+import junit.framework.TestCase;
 
 import org.apache.commons.configuration.reloading.FileAlwaysReloadingStrategy;
 import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
 
-import junit.framework.TestCase;
-
 /**
  * @author Emmanuel Bourg
- * @version $Revision: 561230 $, $Date: 2007-07-31 06:17:09 +0200 (Di, 31 Jul 2007) $
+ * @version $Revision: 711782 $, $Date: 2008-11-06 08:23:41 +0100 (Do, 06 Nov 2008) $
  */
 public class TestFileConfiguration extends TestCase
 {
@@ -426,7 +427,7 @@ public class TestFileConfiguration extends TestCase
     }
 
     /**
-     * Tests to invoke save() without explicitely setting a file name. This
+     * Tests to invoke save() without explicitly setting a file name. This
      * will cause an exception.
      */
     public void testSaveWithoutFileName() throws Exception
@@ -589,6 +590,44 @@ public class TestFileConfiguration extends TestCase
         assertNotNull("Exception is not set", l.getLastEvent().getCause());
     }
 
+    /**
+     * Tests iterating over the keys of a non hierarchical file-based
+     * configuration while a reload happens. This test is related to
+     * CONFIGURATION-347.
+     */
+    public void testIterationWithReloadFlat() throws ConfigurationException
+    {
+        PropertiesConfiguration config = new PropertiesConfiguration(TEST_FILE);
+        checkIterationWithReload(config);
+    }
+
+    /**
+     * Tests iterating over the keys of a hierarchical file-based configuration
+     * while a reload happens. This test is related to CONFIGURATION-347.
+     */
+    public void testIterationWithReloadHierarchical()
+            throws ConfigurationException
+    {
+        XMLConfiguration config = new XMLConfiguration("test.xml");
+        checkIterationWithReload(config);
+    }
+
+    /**
+     * Helper method for testing an iteration over the keys of a file-based
+     * configuration while a reload happens.
+     *
+     * @param config the configuration to test
+     */
+    private void checkIterationWithReload(FileConfiguration config)
+    {
+        config.setReloadingStrategy(new FileAlwaysReloadingStrategy());
+        for (Iterator it = config.getKeys(); it.hasNext();)
+        {
+            String key = (String) it.next();
+            assertNotNull("No value for key " + key, config.getProperty(key));
+        }
+    }
+
     /**
      * Helper method for comparing the content of two configuration objects.
      *
