@@ -24,20 +24,21 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
+import junit.framework.TestCase;
+
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.configuration.HierarchicalConfiguration.Node;
 import org.apache.commons.configuration.event.ConfigurationEvent;
 import org.apache.commons.configuration.event.ConfigurationListener;
+import org.apache.commons.configuration.tree.ConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
 import org.apache.commons.configuration.tree.ExpressionEngine;
 
-import junit.framework.TestCase;
-
 /**
  * Test class for HierarchicalConfiguration.
  *
- * @version $Id: TestHierarchicalConfiguration.java 589136 2007-10-27 15:38:15Z oheger $
+ * @version $Id: TestHierarchicalConfiguration.java 722238 2008-12-01 21:28:31Z oheger $
  */
 public class TestHierarchicalConfiguration extends TestCase
 {
@@ -356,6 +357,35 @@ public class TestHierarchicalConfiguration extends TestCase
                 new String[] {"url", "user" });
     }
 
+    /**
+     * Tests getKeys() with a prefix when the prefix matches exactly a key.
+     */
+    public void testGetKeysWithKeyAsPrefix()
+    {
+        config.addProperty("order.key1", "value1");
+        config.addProperty("order.key2", "value2");
+        Iterator it = config.getKeys("order.key1");
+        assertTrue("no key found", it.hasNext());
+        assertEquals("1st key", "order.key1", it.next());
+        assertFalse("more keys than expected", it.hasNext());
+    }
+
+    /**
+     * Tests getKeys() with a prefix when the prefix matches exactly a key, and
+     * there are multiple keys starting with this prefix.
+     */
+    public void testGetKeysWithKeyAsPrefixMultiple()
+    {
+        config.addProperty("order.key1", "value1");
+        config.addProperty("order.key1.test", "value2");
+        config.addProperty("order.key1.test.complex", "value2");
+        Iterator it = config.getKeys("order.key1");
+        assertEquals("Wrong key 1", "order.key1", it.next());
+        assertEquals("Wrong key 2", "order.key1.test", it.next());
+        assertEquals("Wrong key 3", "order.key1.test.complex", it.next());
+        assertFalse("More keys than expected", it.hasNext());
+    }
+
     public void testAddProperty()
     {
         config.addProperty("tables.table(0).fields.field(-1).name", "phone");
@@ -674,6 +704,20 @@ public class TestHierarchicalConfiguration extends TestCase
         }
     }
 
+    /**
+     * Tests adding an attribute node with the addNodes() method.
+     */
+    public void testAddNodesAttributeNode()
+    {
+        Collection nodes = new ArrayList();
+        ConfigurationNode nd = createNode("length", "10");
+        nd.setAttribute(true);
+        nodes.add(nd);
+        config.addNodes("tables.table(0).fields.field(1)", nodes);
+        assertEquals("Attribute was not added", "10", config
+                .getString("tables.table(0).fields.field(1)[@length]"));
+    }
+
     /**
      * Tests removing children from a configuration node.
      */
@@ -905,6 +949,43 @@ public class TestHierarchicalConfiguration extends TestCase
 		assertTrue("Configuration not empty", copy.isEmpty());
 	}
 
+	/**
+     * Tests the parents of nodes when setRootNode() is involved. This is
+     * related to CONFIGURATION-334.
+     */
+    public void testNodeParentsAfterSetRootNode()
+    {
+        DefaultConfigurationNode root = new DefaultConfigurationNode();
+        DefaultConfigurationNode child1 = new DefaultConfigurationNode(
+                "child1", "test1");
+        root.addChild(child1);
+        config.setRootNode(root);
+        config.addProperty("child2", "test2");
+        List nodes = config.getExpressionEngine().query(config.getRootNode(),
+                "child2");
+        assertEquals("Wrong number of result nodes", 1, nodes.size());
+        ConfigurationNode child2 = (ConfigurationNode) nodes.get(0);
+        assertEquals("Different parent nodes", child1.getParentNode(), child2
+                .getParentNode());
+    }
+
+    /**
+     * Tests calling getRoot() after a root node was set using setRootNode() and
+     * further child nodes have been added. The newly add child nodes should be
+     * present in the root node returned.
+     */
+    public void testGetRootAfterSetRootNode()
+    {
+        DefaultConfigurationNode root = new DefaultConfigurationNode();
+        DefaultConfigurationNode child1 = new DefaultConfigurationNode(
+                "child1", "test1");
+        root.addChild(child1);
+        config.setRootNode(root);
+        config.addProperty("child2", "test2");
+        ConfigurationNode oldRoot = config.getRoot();
+        assertEquals("Wrong number of children", 2, oldRoot.getChildrenCount());
+    }
+
 	/**
      * Helper method for testing the getKeys(String) method.
      *
