@@ -21,6 +21,7 @@ import java.io.File;
 import java.io.FileOutputStream;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.Reader;
@@ -34,14 +35,15 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
-import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
-
 import junit.framework.TestCase;
 
+import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
+import org.apache.commons.lang.SystemUtils;
+
 /**
  * Test for loading and saving properties files.
  *
- * @version $Id: TestPropertiesConfiguration.java 567771 2007-08-20 17:57:08Z oheger $
+ * @version $Id: TestPropertiesConfiguration.java 712431 2008-11-08 21:05:23Z oheger $
  */
 public class TestPropertiesConfiguration extends TestCase
 {
@@ -112,6 +114,16 @@ public class TestPropertiesConfiguration extends TestCase
         assertEquals("true", loaded);
     }
 
+    /**
+     * test if includes properties from interpolated file
+     * name get loaded
+     */
+    public void testLoadIncludeInterpol() throws Exception
+    {
+        String loaded = conf.getString("include.interpol.loaded");
+        assertEquals("true", loaded);
+    }
+
     public void testSetInclude() throws Exception
     {
         // change the include key
@@ -739,6 +751,24 @@ public class TestPropertiesConfiguration extends TestCase
         }
     }
 
+    /**
+     * Test the creation of a file containing a '#' in its name. This test is
+     * skipped on Java 1.3 as it always fails.
+     */
+    public void testFileWithSharpSymbol() throws Exception
+    {
+        if (SystemUtils.isJavaVersionAtLeast(1.4f))
+        {
+            File file = new File("target/sharp#1.properties");
+            file.createNewFile();
+
+            PropertiesConfiguration conf = new PropertiesConfiguration(file);
+            conf.save();
+
+            assertTrue("Missing file " + file, file.exists());
+        }
+    }
+
     /**
      * Tests initializing a properties configuration from a non existing file.
      * There was a bug, which caused properties getting lost when later save()
@@ -779,6 +809,53 @@ public class TestPropertiesConfiguration extends TestCase
         checkCopiedConfig(copyConf);
     }
 
+    /**
+     * Tests adding properties through a DataConfiguration. This is related to
+     * CONFIGURATION-332.
+     */
+    public void testSaveWithDataConfig() throws ConfigurationException
+    {
+        conf = new PropertiesConfiguration(testSavePropertiesFile);
+        DataConfiguration dataConfig = new DataConfiguration(conf);
+        dataConfig.setProperty("foo", "bar");
+        assertEquals("Property not set", "bar", conf.getString("foo"));
+
+        conf.save();
+        PropertiesConfiguration config2 = new PropertiesConfiguration(
+                testSavePropertiesFile);
+        assertEquals("Property not saved", "bar", config2.getString("foo"));
+    }
+
+    /**
+     * Tests whether the correct default encoding is used when loading a
+     * properties file. This test is related to CONFIGURATION-345.
+     */
+    public void testLoadWithDefaultEncoding() throws ConfigurationException
+    {
+        class PropertiesConfigurationTestImpl extends PropertiesConfiguration
+        {
+            String loadEncoding;
+
+            public PropertiesConfigurationTestImpl(String fileName)
+                    throws ConfigurationException
+            {
+                super(fileName);
+            }
+
+            public void load(InputStream in, String encoding)
+                    throws ConfigurationException
+            {
+                loadEncoding = encoding;
+                super.load(in, encoding);
+            }
+        }
+
+        PropertiesConfigurationTestImpl testConf = new PropertiesConfigurationTestImpl(
+                testProperties);
+        assertEquals("Default encoding not used", "ISO-8859-1",
+                testConf.loadEncoding);
+    }
+
     /**
      * Creates a configuration that can be used for testing copy operations.
      *
