@@ -25,7 +25,7 @@ import junit.framework.TestCase;
  * Tests for MapConfiguration.
  *
  * @author Emmanuel Bourg
- * @version $Revision: 439648 $, $Date: 2006-09-02 22:42:10 +0200 (Sa, 02 Sep 2006) $
+ * @version $Revision: 727834 $, $Date: 2008-12-18 23:16:32 +0100 (Do, 18 Dez 2008) $
  */
 public class TestSystemConfiguration extends TestCase
 {
@@ -37,4 +37,13 @@ public class TestSystemConfiguration extends TestCase
         Configuration conf = new SystemConfiguration();
         assertEquals("number", 123, conf.getInt("test.number"));
     }
+
+    public void testSetSystemProperties()
+    {
+        PropertiesConfiguration props = new PropertiesConfiguration();
+        props.addProperty("test.name", "Apache");
+        SystemConfiguration conf = new SystemConfiguration();
+        conf.setSystemProperties(props);
+        assertEquals("System Properties", "Apache", System.getProperty("test.name"));       
+    }
 }
