@@ -47,7 +47,7 @@ import org.xml.sax.helpers.DefaultHandler;
 /**
  * test for loading and saving xml properties files
  *
- * @version $Id: TestXMLConfiguration.java 578579 2007-09-23 16:20:24Z oheger $
+ * @version $Id: TestXMLConfiguration.java 721895 2008-11-30 21:08:42Z oheger $
  */
 public class TestXMLConfiguration extends TestCase
 {
@@ -963,6 +963,26 @@ public class TestXMLConfiguration extends TestCase
                         .getString("entity"));
     }
 
+    /**
+     * Tests whether reloads are recognized when querying the configuration's
+     * keys.
+     */
+    public void testGetKeysWithReload() throws ConfigurationException
+    {
+        XMLConfiguration c = setUpReloadTest();
+        conf.addProperty("aNewKey", "aNewValue");
+        conf.save(testSaveConf);
+        boolean found = false;
+        for (Iterator it = c.getKeys(); it.hasNext();)
+        {
+            if ("aNewKey".equals(it.next()))
+            {
+                found = true;
+            }
+        }
+        assertTrue("Reload not performed", found);
+    }
+
     /**
      * Tests accessing properties when the XPATH expression engine is set.
      */
@@ -997,6 +1017,36 @@ public class TestXMLConfiguration extends TestCase
         checkSavedConfig(copy);
     }
 
+    /**
+     * Tests setting text of the root element.
+     */
+    public void testSetTextRootElement() throws ConfigurationException
+    {
+        conf.setProperty("", "Root text");
+        conf.save(testSaveConf);
+        XMLConfiguration copy = new XMLConfiguration();
+        copy.setFile(testSaveConf);
+        checkSavedConfig(copy);
+    }
+
+    /**
+     * Tests removing the text of the root element.
+     */
+    public void testClearTextRootElement() throws ConfigurationException
+    {
+        final String xml = "<e a=\"v\">text</e>";
+        conf.clear();
+        StringReader in = new StringReader(xml);
+        conf.load(in);
+        assertEquals("Wrong text of root", "text", conf.getString(""));
+
+        conf.clearProperty("");
+        conf.save(testSaveConf);
+        XMLConfiguration copy = new XMLConfiguration();
+        copy.setFile(testSaveConf);
+        checkSavedConfig(copy);
+    }
+
     /**
      * Tests list nodes with multiple values and attributes.
      */
@@ -1278,6 +1328,153 @@ public class TestXMLConfiguration extends TestCase
         }
     }
 
+    /**
+     * Tests saving a configuration that was created from a hierarchical
+     * configuration. This test exposes bug CONFIGURATION-301.
+     */
+    public void testSaveAfterCreateWithCopyConstructor()
+            throws ConfigurationException
+    {
+        HierarchicalConfiguration hc = conf.configurationAt("element2");
+        conf = new XMLConfiguration(hc);
+        conf.save(testSaveConf);
+        XMLConfiguration checkConfig = new XMLConfiguration();
+        checkConfig.setFile(testSaveConf);
+        checkSavedConfig(checkConfig);
+        assertEquals("Wrong name of root element", "element2", checkConfig
+                .getRootElementName());
+    }
+
+    /**
+     * Tests whether the name of the root element is copied when a configuration
+     * is created using the copy constructor.
+     */
+    public void testCopyRootName() throws ConfigurationException
+    {
+        final String rootName = "rootElement";
+        final String xml = "<" + rootName + "><test>true</test></" + rootName
+                + ">";
+        conf.clear();
+        conf.load(new StringReader(xml));
+        XMLConfiguration copy = new XMLConfiguration(conf);
+        assertEquals("Wrong name of root element", rootName, copy
+                .getRootElementName());
+        copy.save(testSaveConf);
+        copy = new XMLConfiguration(testSaveConf);
+        assertEquals("Wrong name of root element after save", rootName, copy
+                .getRootElementName());
+    }
+
+    /**
+     * Tests whether the name of the root element is copied for a configuration
+     * for which not yet a document exists.
+     */
+    public void testCopyRootNameNoDocument() throws ConfigurationException
+    {
+        final String rootName = "rootElement";
+        conf = new XMLConfiguration();
+        conf.setRootElementName(rootName);
+        conf.setProperty("test", Boolean.TRUE);
+        XMLConfiguration copy = new XMLConfiguration(conf);
+        assertEquals("Wrong name of root element", rootName, copy
+                .getRootElementName());
+        copy.save(testSaveConf);
+        copy = new XMLConfiguration(testSaveConf);
+        assertEquals("Wrong name of root element after save", rootName, copy
+                .getRootElementName());
+    }
+
+    /**
+     * Tests adding an attribute node using the addNodes() method.
+     */
+    public void testAddNodesAttributeNode()
+    {
+        conf.addProperty("testAddNodes.property[@name]", "prop1");
+        conf.addProperty("testAddNodes.property(0).value", "value1");
+        conf.addProperty("testAddNodes.property(-1)[@name]", "prop2");
+        conf.addProperty("testAddNodes.property(1).value", "value2");
+        Collection nodes = new ArrayList();
+        nodes.add(new HierarchicalConfiguration.Node("property"));
+        conf.addNodes("testAddNodes", nodes);
+        nodes.clear();
+        ConfigurationNode nd = new HierarchicalConfiguration.Node("name",
+                "prop3");
+        nd.setAttribute(true);
+        nodes.add(nd);
+        conf.addNodes("testAddNodes.property(2)", nodes);
+        assertEquals("Attribute not added", "prop3", conf
+                .getString("testAddNodes.property(2)[@name]"));
+    }
+
+    /**
+     * Tests whether spaces are preserved when the xml:space attribute is set.
+     */
+    public void testPreserveSpace()
+    {
+        assertEquals("Wrong value of blanc", " ", conf.getString("space.blanc"));
+        assertEquals("Wrong value of stars", " * * ", conf
+                .getString("space.stars"));
+    }
+
+    /**
+     * Tests whether the xml:space attribute can be overridden in nested
+     * elements.
+     */
+    public void testPreserveSpaceOverride()
+    {
+        assertEquals("Not trimmed", "Some text", conf
+                .getString("space.description"));
+    }
+
+    /**
+     * Tests an xml:space attribute with an invalid value. This will be
+     * interpreted as default.
+     */
+    public void testPreserveSpaceInvalid()
+    {
+        assertEquals("Invalid not trimmed", "Some other text", conf
+                .getString("space.testInvalid"));
+    }
+
+    /**
+     * Tests whether attribute splitting can be disabled.
+     */
+    public void testAttributeSplittingDisabled() throws ConfigurationException
+    {
+        List values = conf.getList("expressions[@value2]");
+        assertEquals("Wrong number of attribute values", 2, values.size());
+        assertEquals("Wrong value 1", "a", values.get(0));
+        assertEquals("Wrong value 2", "b|c", values.get(1));
+        XMLConfiguration conf2 = new XMLConfiguration();
+        conf2.setAttributeSplittingDisabled(true);
+        conf2.setFile(conf.getFile());
+        conf2.load();
+        assertEquals("Attribute was split", "a,b|c", conf2
+                .getString("expressions[@value2]"));
+    }
+
+    /**
+     * Tests disabling both delimiter parsing and attribute splitting.
+     */
+    public void testAttributeSplittingAndDelimiterParsingDisabled()
+            throws ConfigurationException
+    {
+        conf.clear();
+        conf.setDelimiterParsingDisabled(true);
+        conf.load();
+        List values = conf.getList("expressions[@value2]");
+        assertEquals("Wrong number of attribute values", 2, values.size());
+        assertEquals("Wrong value 1", "a,b", values.get(0));
+        assertEquals("Wrong value 2", "c", values.get(1));
+        XMLConfiguration conf2 = new XMLConfiguration();
+        conf2.setAttributeSplittingDisabled(true);
+        conf2.setDelimiterParsingDisabled(true);
+        conf2.setFile(conf.getFile());
+        conf2.load();
+        assertEquals("Attribute was split", "a,b|c", conf2
+                .getString("expressions[@value2]"));
+    }
+
     /**
      * Prepares a configuration object for testing a reload operation.
      *
