@@ -32,7 +32,7 @@ import org.apache.commons.configuration.ConfigurationComparator;
 
 /**
  * @author Emmanuel Bourg
- * @version $Revision: 530587 $, $Date: 2007-04-20 01:08:36 +0200 (Fr, 20 Apr 2007) $
+ * @version $Revision: 645568 $, $Date: 2008-04-07 17:37:45 +0200 (Mo, 07 Apr 2008) $
  */
 public class TestXMLPropertyListConfiguration extends TestCase
 {
@@ -52,7 +52,7 @@ public class TestXMLPropertyListConfiguration extends TestCase
 
     public void testInteger() throws Exception
     {
-        assertEquals("'integer' property", 12345, config.getInt("integer"));
+        assertEquals("'integer' property", 12345678900L, config.getLong("integer"));
     }
 
     public void testReal() throws Exception
@@ -73,6 +73,21 @@ public class TestXMLPropertyListConfiguration extends TestCase
         assertEquals("3rd element", "value3", config.getProperty("dictionary.key3"));
     }
 
+    public void testDate() throws Exception
+    {
+        Calendar calendar = Calendar.getInstance();
+        calendar.clear();
+        calendar.setTimeZone(TimeZone.getTimeZone("UTC"));
+        calendar.set(2005, Calendar.JANUARY, 1, 12, 0, 0);
+
+        assertEquals("'date' property", calendar.getTime(), config.getProperty("date"));
+
+        calendar.setTimeZone(TimeZone.getTimeZone("CET"));
+        calendar.set(2002, Calendar.MARCH, 22, 11, 30, 0);
+
+        assertEquals("'date-gnustep' property", calendar.getTime(), config.getProperty("date-gnustep"));
+    }
+
     public void testSubset()
     {
         Configuration subset = config.subset("dictionary");
